import os
import shutil
import unittest

from scconfig.config import Config
from .sample_config.default import DEFAULT_CONFIG


class ConfigTestCase(unittest.TestCase):

    def test_create_config(self):
        config = Config.create(project_name="sc-config")
        self.assertIsNotNone(config)
        environment = config.get("environment")

        self.assertEqual(environment, 'production')

    def test_create_config_with_defaults(self):
        config = Config.create(project_name="sc-config", defaults=DEFAULT_CONFIG)
        self.assertIsNotNone(config)
        environment = config.get("environment")

        self.assertEqual(environment, 'production')
        self.assertEqual(config.get("dev.dev_mode"), True)

    def test_create_config_from_project_file(self):
        project_name = "sc-config"
        config_directory = '.{}'.format(project_name)
        directory = os.path.join(os.path.expanduser('~'), config_directory)
        if not os.path.exists(directory):
            os.makedirs(directory)

        with open("tests/sample_config/development.yml", "r") as reader:
            dev_file_content = reader.readlines()
        filename = Config._get_config_file_path(project_name, "development")
        with open(filename, "w+") as writer:
            writer.writelines(dev_file_content)

        config = Config.create(project_name=project_name, environment="development")
        self.assertIsNotNone(config)
        environment = config.get("environment")

        self.assertEqual(environment, 'development')
        self.assertEqual(config.get("dev.dev_mode"), True)

        try:
            shutil.rmtree(directory)
        except OSError as e:
            print("failed to delete directory: %s : %s" % (directory, e.strerror))


if __name__ == '__main__':
    unittest.main()



"""Configurations

Copyright (c) 2021 Scott Lau
"""

import logging
import os

from config42 import ConfigManager


class Config:

    @staticmethod
    def _get_config_file_path(project_name, environment):
        config_directory = '.{}'.format(project_name)
        return os.path.join(os.path.expanduser('~'), '{}/{}.yml'.format(config_directory, environment))

    @staticmethod
    def create(*, project_name, environment=None, defaults={}):
        prefix = project_name.upper()
        prefix = prefix.replace("-", "_")
        # load defaults from defaults.py file
        config = ConfigManager(defaults=defaults)
        # load defaults from home directory
        config_file = Config._get_config_file_path(project_name, "default")
        if os.path.exists(config_file):
            logging.getLogger(__name__).info("loading default configurations from %s", config_file)
            config.set_many(ConfigManager(path=config_file).as_dict())
        # load environment configurations from environment variables
        env_config = ConfigManager(prefix=prefix)
        config.set_many(env_config.as_dict())

        key_env = "environment"
        if environment is None:
            environment = env_config.get(key_env)
            if environment is None:
                # use production configuration if not specified environment
                environment = "production"
                logging.getLogger(__name__).info("did not specify environment, using %s", environment)
        else:
            logging.getLogger(__name__).info("using environment: %s", environment)
        config.set(key_env, environment)
        # load environment configurations from file
        env_config_file = Config._get_config_file_path(project_name, environment)
        if os.path.exists(env_config_file):
            logging.getLogger(__name__).info("loading environmental configurations from %s", env_config_file)
            config.set_many(ConfigManager(path=env_config_file).as_dict())
        return config



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Setup

Copyright (c) 2021 Scott Lau
"""

import os

from setuptools import setup, find_packages


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError("Unable to find version string.")


with open('README.md') as fd:
    readme = fd.read()

setup(
    name='sc-config',
    version=get_version("scconfig/__init__.py"),
    url='https://github.com/Scott-Lau/sc-config',
    packages=find_packages(),
    author='Scott Lau',
    author_email='exceedego@126.com',
    license='MIT',
    platforms='POSIX',
    description='A simple Python configuration file operator',
    long_description=readme,
    keywords=(
        'python config'
    ),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Topic :: Software Development',
        'Topic :: Utilities',
    ],
    python_requires='>=3.6',
    install_requires=[
        'config42>=0.4.4',
    ],
)
